package com.sleepycat.je.dbi;

import java.io.PrintStream;

import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseEntry;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.OperationStatus;
import com.sleepycat.je.VerifyConfig;
import com.sleepycat.je.tree.MapLN;
import com.sleepycat.je.tree.Node;
import com.sleepycat.je.txn.BasicLocker;
import com.sleepycat.je.txn.LockType;
import com.sleepycat.je.txn.Locker;

public privileged aspect Verifier {
	/*
	 * @param config verify configuration @param emptyStats empty database
	 * stats, to be filled by this method @return true if the verify saw no
	 * errors.
	 */
	public boolean DatabaseImpl.verify(VerifyConfig config)
			throws DatabaseException {

		if (tree == null) {
			return true;
		}

		PrintStream out = config.getShowProgressStream();
		if (out == null) {
			out = System.err;
		}

		TreeVerifier verifier = new TreeVerifier(out);
		boolean ok = walkDatabaseTree(verifier, out, config.getPrintInfo());
		return ok;
	}

	/*
	 * Verification, must be run while system is quiescent.
	 */
	public boolean DbTree.verify(VerifyConfig config, PrintStream out)
			throws DatabaseException {

		boolean ret = true;
		try {
			/* For now, verify all databases. */
			boolean ok = idDatabase.verify(config);
			if (!ok) {
				ret = false;
			}

			ok = nameDatabase.verify(config);
			if (!ok) {
				ret = false;
			}
		} catch (DatabaseException DE) {
			ret = false;
		}

		synchronized (envImpl.getINCompressor()) {

			/*
			 * Get a cursor on the id tree. Use objects at the dbi layer rather
			 * than at the public api, in order to retrieve objects rather than
			 * Dbts. Note that we don't do cursor cloning here, so any failures
			 * from each db verify invalidate the cursor. Use dirty read
			 * (LockMode.NONE) because locks on the MapLN should never be held
			 * for long, as that will cause deadlocks with splits and
			 * checkpointing.
			 */
			Locker locker = null;
			CursorImpl cursor = null;
			LockType lockType = LockType.NONE;
			try {
				locker = new BasicLocker(envImpl);
				cursor = new CursorImpl(idDatabase, locker);
				if (cursor.positionFirstOrLast(true, null)) {
					MapLN mapLN = (MapLN) cursor.getCurrentLN(lockType);

					DatabaseEntry keyDbt = new DatabaseEntry();
					DatabaseEntry dataDbt = new DatabaseEntry();
					while (true) {
						if (mapLN != null && !mapLN.isDeleted()) {
							DatabaseImpl dbImpl = mapLN.getDatabase();
							boolean ok = dbImpl.verify(config);
							if (!ok) {
								ret = false;
							}
						}
						/* Go on to the next entry. */
						OperationStatus status = cursor.getNext(keyDbt,
								dataDbt, lockType, true // go forward
								/* Lck false */); // do need to latch
						if (status != OperationStatus.SUCCESS) {
							break;
						}
						cursor.hook_latchBIN(); // Lck needed to introduce to
												// not to distinguish between
												// latched and not lached calls
												// within this method. so both
												// calls are already latched
						mapLN = (MapLN) cursor.getCurrentLN(lockType);
					}
				}
			} catch (DatabaseException e) {
				e.printStackTrace(out);
				ret = false;
			} finally {
				if (cursor != null) {
					cursor.close();
				}
				if (locker != null) {
					locker.operationEnd();
				}
			}
		}

		return ret;
	}

	/*
	 * Verification, must be run while system is quiescent.
	 */
	public boolean EnvironmentImpl.verify(VerifyConfig config, PrintStream out)
			throws DatabaseException {

		/* For now, verify all databases */
		return dbMapTree.verify(config, out);
	}

	// changed signature to seperate verify and stats concerns
	public boolean Database.verify(VerifyConfig config)
			throws DatabaseException {

		checkEnv();
		checkRequiredDbState(OPEN, "Can't call Database.verify");
		databaseImpl.checkIsDeleted("verify");
		VerifyConfig useConfig = (config == null) ? VerifyConfig.DEFAULT
				: config;

		return databaseImpl.verify(useConfig);
	}
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public boolean Environment.verify(VerifyConfig config, PrintStream out)
			throws DatabaseException {

		checkHandleIsValid();
		checkEnv();
		VerifyConfig useConfig = (config == null) ? VerifyConfig.DEFAULT
				: config;
		return environmentImpl.verify(useConfig, out);
	}
}
